const net = require('net');

// Datos del MasterServer
const masterserverIp = 'localhost'; // Cambiar por la IP del masterserver
const masterserverPort = 3000; // Puerto del masterserver

// Datos del servidor de chat
const serverIp = '0.0.0.0'; // Cambiar por la IP Publica(externa) del servidor de chat.
const listenPort = 5555; // Puerto del servidor de chat para escuchar conexiones.
const sendListenPort = 5555; // Puerto publico(externo) del servidor de chat(Cambiarlo si es diferente).
const serverName = 'Mi servidor de chat'; // Nombre del servidor de chat.
let useMasterServer = false; // Puedes cambiar esto a false para desactivar la conexión con el Master Server

// Array para almacenar los clientes conectados
let clients = [];
let masterConnected = false;
let reconnectInterval;

const emptyness = ''

// Función para conectar al MasterServer
function connectToMaster() {
    if (useMasterServer && !masterConnected) {
        console.log('Intentando conectar al MasterServer...');
        masterClient.connect({ port: masterserverPort, host: masterserverIp });
    }
}

// Crear un cliente TCP para conectarse al MasterServer
const masterClient = new net.Socket();

// Manejar la conexión con el MasterServer
masterClient.on('connect', () => {
    console.log('Conexión establecida con el MasterServer');
    masterConnected = true;
    clearInterval(reconnectInterval);
    // Enviar la información al MasterServer
    masterClient.write(JSON.stringify({ ip: serverIp, port: sendListenPort, name: serverName }));
});

// Manejar los datos recibidos del MasterServer
masterClient.on('data', (data) => {
    console.log('Respuesta del MasterServer:', data.toString());
});

// Manejar la desconexión con el MasterServer
masterClient.on('close', () => {
    console.log('Conexión con el MasterServer cerrada');
    masterConnected = false;
    // Intentar reconexión cada minuto si useMasterServer es verdadero
    if (useMasterServer) {
        reconnectInterval = setInterval(connectToMaster, 60000);
    }
});

// Manejar el error en la conexión con el MasterServer
masterClient.on('error', (error) => {
    console.error('Error en la conexión con el MasterServer:', error.message);
    masterConnected = false;
    // Intentar reconexión cada minuto si useMasterServer es verdadero
    if (useMasterServer) {
        reconnectInterval = setInterval(connectToMaster, 60000);
    }
});

// Crear un servidor TCP
const server = net.createServer(socket => {
    // Agregar el nuevo cliente al array
    clients.push(socket);

    // Manejar los datos recibidos del cliente
    socket.on('data', data => {
        const message = `${socket.userName || emptyness}:${data}`;
        broadcast(message, socket);
    });

    // Manejar el evento 'close' del cliente
    socket.on('close', () => {
        // Eliminar el cliente desconectado del array
        clients = clients.filter(client => client !== socket);
    });

    // Manejar el evento 'error' del cliente
    socket.on('error', error => {
        console.error(`Error en el cliente ${socket.remoteAddress}:${socket.remotePort}: ${error.message}`);
    });

    // Manejar el evento 'username' del cliente
    socket.on('username', userName => {
        socket.userName = userName;
    });
});

// Función para difundir un mensaje a todos los clientes excepto al remitente
function broadcast(message, sender) {
    clients.forEach(client => {
        if (client !== sender && !client.destroyed) {
            client.write(message + '\n'); // Agregar un salto de línea al final del mensaje
        }
    });
    process.stdout.write(message + '\n'); // Imprimir el mensaje en la consola del servidor con un salto de línea
}

// Manejar el evento 'error' del servidor
server.on('error', error => {
    console.error(`Error en el servidor: ${error.message}`);
});

// Escuchar en el puerto listenPort
server.listen(listenPort, () => {
    console.log(`Servidor de chat escuchando en el puerto ${listenPort}`);
});

// Intentar conectar al MasterServer al inicio si useMasterServer es verdadero
if (useMasterServer) {
    connectToMaster();
}
